@@ -36,18 +36,18 @@ static void __litl_write_add_trace_header(litl_write_trace_t* trace) {
     exit(EXIT_FAILURE);
   }
   trace->header = trace->header_ptr;
-
+  memset(trace->header_ptr, 0, trace->header_size);
   if (uname(&uts) < 0)
     perror("Could not use uname()!");
 
   // add a general header
   // version of LiTL
   sprintf((char*) ((litl_general_header_t *) trace->header)->litl_ver, "%s",
-          VERSION);
+	  VERSION);
   // system information
   sprintf((char*) ((litl_general_header_t *) trace->header)->sysinfo,
-          "%s %s %s %s %s", uts.sysname, uts.nodename, uts.release, uts.version,
-          uts.machine);
+	  "%s %s %s %s %s", uts.sysname, uts.nodename, uts.release, uts.version,
+	  uts.machine);
   // a number of processes
   ((litl_general_header_t *) trace->header)->nb_processes = 1;
   // move pointer
@@ -58,7 +58,7 @@ static void __litl_write_add_trace_header(litl_write_trace_t* trace) {
   char* filename = strrchr(trace->filename, '/');
   filename++;
   sprintf((char*) ((litl_process_header_t *) trace->header)->process_name, "%s",
-          filename);
+	  filename);
   ((litl_process_header_t *) trace->header)->nb_threads = trace->nb_threads;
   ((litl_process_header_t *) trace->header)->header_nb_threads =
     trace->nb_threads;
@@ -74,13 +74,6 @@ static void __litl_write_add_trace_header(litl_write_trace_t* trace) {
   trace->header += sizeof(litl_process_header_t);
 }
 
-/*
- * To create trace->buffer_ptr and trace->buffer
- */
-static void __litl_write_init_var(litl_write_trace_t* trace) {
-  pthread_key_create(&trace->index, NULL );
-}
-
 /*
  * Initializes the trace buffer
  */
@@ -133,13 +126,7 @@ litl_write_trace_t* litl_write_init_trace(const litl_size_t buf_size) {
   // initialize the timing mechanism
   litl_time_initialize();
 
-  // a jump function is needed 'cause it is not possible to pass args to
-  //   the calling function through pthread_once
-  void __init() {
-    __litl_write_init_var(trace);
-  }
-  trace->index_once = (pthread_once_t) PTHREAD_ONCE_INIT;
-  pthread_once(&trace->index_once, __init);
+  assert(pthread_key_create(&trace->index, NULL ) == 0);
 
   // set trace->allow_buffer_flush using the environment variable.
   //   By default the buffer flushing is disabled
@@ -187,7 +174,7 @@ static litl_size_t __litl_write_get_header_size(litl_write_trace_t* trace) {
  * Computes the size of data in buffer
  */
 static litl_size_t __litl_write_get_buffer_size(litl_write_trace_t* trace,
-                                                litl_med_size_t pos) {
+						litl_med_size_t pos) {
   return (trace->buffers[pos]->buffer - trace->buffers[pos]->buffer_ptr);
 }
 
@@ -256,9 +243,9 @@ void litl_write_set_filename(litl_write_trace_t* trace, char* filename) {
   if (trace->filename) {
     if (trace->is_header_flushed)
       fprintf(
-          stderr,
-          "Warning: changing the trace file name to %s after some events have been saved in file %s\n",
-          filename, trace->filename);
+	  stderr,
+	  "Warning: changing the trace file name to %s after some events have been saved in file %s\n",
+	  filename, trace->filename);
     free(trace->filename);
   }
 
@@ -276,20 +263,18 @@ void litl_write_set_filename(litl_write_trace_t* trace, char* filename) {
  * Records an event with offset only
  */
 static void __litl_write_probe_offset(litl_write_trace_t* trace,
-                                      litl_med_size_t index) {
+				      litl_med_size_t index) {
   if (!trace->is_litl_initialized || trace->is_recording_paused)
     return;
 
-  // litl_t* cur_ptr = litl_cmpxchg((uint8_t**) &trace->buffer[index],
-  //    LITL_BASE_SIZE + sizeof(litl_param_t));
   litl_t* cur_ptr = (litl_t *) trace->buffers[index]->buffer;
-
   cur_ptr->time = 0;
   cur_ptr->code = LITL_OFFSET_CODE;
   cur_ptr->type = LITL_TYPE_REGULAR;
   cur_ptr->parameters.offset.nb_params = 1;
   cur_ptr->parameters.offset.offset = 0;
-  trace->buffers[index]->buffer += LITL_BASE_SIZE + sizeof(litl_param_t);
+
+  trace->buffers[index]->buffer += __litl_get_gen_event_size(cur_ptr);
 }
 
 /* Open the trace file. If the file already exists, delete it first
@@ -318,19 +303,25 @@ static void __litl_open_new_file(litl_write_trace_t* trace) {
 }
 
 /*
- * Writes the recorded events from the buffer to the trace file
+ * Write the header on the disk
  */
-static void __litl_write_flush_buffer(litl_write_trace_t* trace,
-                                      litl_med_size_t index) {
-  int res __attribute__ ((__unused__));
-  litl_offset_t offset, header_size;
+static void __litl_write_update_header(litl_write_trace_t* trace) {
+  // write the trace header to the trace file
+  assert(trace->f_handle);
+  lseek(trace->f_handle, 0, SEEK_SET);
 
-  if (!trace->is_litl_initialized)
-    return;
-
-  if (trace->allow_thread_safety)
-    pthread_mutex_lock(&trace->lock_litl_flush);
+  if (write(trace->f_handle, trace->header_ptr,
+	    __litl_write_get_header_size(trace)) == -1) {
+    perror(
+	   "Flushing the buffer. Could not write measured data to the trace file!");
+    exit(EXIT_FAILURE);
+  }
+}
 
+/*
+ * Update the header and flush it to disk
+ */
+static void __litl_write_flush_header(litl_write_trace_t* trace) {
   if (!trace->is_header_flushed) {
     // open the trace file
     __litl_open_new_file(trace);
@@ -351,7 +342,7 @@ static void __litl_write_flush_buffer(litl_write_trace_t* trace,
 
       // save the position of offset inside the trace file
       trace->buffers[i]->offset = __litl_write_get_header_size(trace)
-        - sizeof(litl_offset_t);
+	- sizeof(litl_offset_t);
       trace->buffers[i]->already_flushed = 1;
     }
 
@@ -360,17 +351,13 @@ static void __litl_write_flush_buffer(litl_write_trace_t* trace,
     trace->header_offset = __litl_write_get_header_size(trace);
 
     // specify the last slot of pairs (offset == 0)
-    ((litl_thread_pair_t *) trace->header)->tid = 0;
-    ((litl_thread_pair_t *) trace->header)->offset = 0;
+    litl_thread_pair_t *thread_pair = (litl_thread_pair_t *) trace->header;
     trace->header += sizeof(litl_thread_pair_t);
+    thread_pair->tid = 0;
+    thread_pair->offset = 0;
 
     // write the trace header to the trace file
-    if (write(trace->f_handle, trace->header_ptr,
-              __litl_write_get_header_size(trace)) == -1) {
-      perror(
-          "Flushing the buffer. Could not write measured data to the trace file!");
-      exit(EXIT_FAILURE);
-    }
+    __litl_write_update_header(trace);
 
     trace->general_offset = __litl_write_get_header_size(trace);
 
@@ -380,65 +367,110 @@ static void __litl_write_flush_buffer(litl_write_trace_t* trace,
 
     trace->is_header_flushed = 1;
   }
+}
+
+
+/*
+ * Write the thread-specific header to disk
+ */
+static void __litl_write_flush_thread_header(litl_write_trace_t* trace,
+					     litl_med_size_t index,
+					     litl_offset_t header_size) {
+  litl_offset_t offset;
+  int res;
+  // when more buffers to store threads information is required
+  if (trace->nb_threads
+      > (trace->header_nb_threads + NBTHREADS * trace->nb_slots)) {
+
+    // updated the offset from the previous slot
+    lseek(trace->f_handle, trace->header_offset + sizeof(litl_tid_t),
+	  SEEK_SET);
+    offset = trace->general_offset - header_size;
+    res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
+    assert(res>=0);
+
+    // reserve a new slot for pairs (tid, offset)
+    trace->header_offset = trace->general_offset;
+    trace->threads_offset = trace->header_offset;
+    trace->general_offset += (NBTHREADS + 1) * sizeof(litl_thread_pair_t);
+
+    trace->nb_slots++;
+  }
+
+  // add a new pair (tid, offset)
+  lseek(trace->f_handle, trace->header_offset, SEEK_SET);
+  res = write(trace->f_handle, &trace->buffers[index]->tid,
+	      sizeof(litl_tid_t));
+  assert(res >= 0);
+  offset = trace->general_offset - header_size;
+  res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
+  assert(res >= 0);
+
+  // add an indicator to specify the last slot of pairs (offset == 0)
+  // TODO: how to optimize this and write only once at the end of the slot
+  offset = 0;
+  res = write(trace->f_handle, &offset, sizeof(litl_tid_t));
+  assert(res >= 0);
+  res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
+  assert(res >= 0);
+
+  trace->header_offset += sizeof(litl_thread_pair_t);
+  trace->buffers[index]->already_flushed = 1;
+
+  // updated the number of threads
+  // TODO: perform update only once 'cause there is duplication
+  lseek(trace->f_handle, trace->header_size, SEEK_SET);
+  res = write(trace->f_handle, &trace->nb_threads, sizeof(litl_med_size_t));
+  assert(res >= 0);
+}
+
+/*
+ * Update the thread-specific header and write it to disk
+ */
+static void __litl_write_update_thread_header(litl_write_trace_t* trace,
+					      litl_med_size_t index,
+					      litl_offset_t header_size) {
+    // update the previous offset of the current thread,
+    //   updating the location in the file
+    lseek(trace->f_handle, trace->buffers[index]->offset, SEEK_SET);
+    litl_offset_t offset = trace->general_offset - header_size;
+    int res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
+    assert(res >= 0);
+}
+
+/*
+ * Writes the recorded events from the buffer to the trace file
+ */
+static void __litl_write_flush_buffer(litl_write_trace_t* trace,
+				      litl_med_size_t index) {
+  int res __attribute__ ((__unused__));
+  litl_offset_t header_size;
+  if (!trace->is_litl_initialized)
+    return;
+
+  if (trace->allow_thread_safety)
+    pthread_mutex_lock(&trace->lock_litl_flush);
+
+  if (!trace->is_header_flushed) {
+    /* flush the header to disk */
+    __litl_write_flush_header(trace);
+  }
 
   header_size = sizeof(litl_general_header_t) + sizeof(litl_process_header_t);
   // handle the situation when some threads start after the header was flushed
   if (!trace->buffers[index]->already_flushed) {
-
-    // when more buffers to store threads information is required
-    if (trace->nb_threads
-      > (trace->header_nb_threads + NBTHREADS * trace->nb_slots)) {
-
-      // updated the offset from the previous slot
-      lseek(trace->f_handle, trace->header_offset + sizeof(litl_tid_t),
-            SEEK_SET);
-      offset = trace->general_offset - header_size;
-      res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
-
-      // reserve a new slot for pairs (tid, offset)
-      trace->header_offset = trace->general_offset;
-      trace->threads_offset = trace->header_offset;
-      trace->general_offset += (NBTHREADS + 1) * sizeof(litl_thread_pair_t);
-
-      trace->nb_slots++;
-    }
-
-    // add a new pair (tid, offset)
-    lseek(trace->f_handle, trace->header_offset, SEEK_SET);
-    res = write(trace->f_handle, &trace->buffers[index]->tid,
-                sizeof(litl_tid_t));
-    offset = trace->general_offset - header_size;
-    res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
-
-    // add an indicator to specify the last slot of pairs (offset == 0)
-    // TODO: how to optimize this and write only once at the end of the slot
-    offset = 0;
-    res = write(trace->f_handle, &offset, sizeof(litl_tid_t));
-    res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
-
-    trace->header_offset += sizeof(litl_thread_pair_t);
-    trace->buffers[index]->already_flushed = 1;
-
-    // updated the number of threads
-    // TODO: perform update only once 'cause there is duplication
-    lseek(trace->f_handle, trace->header_size, SEEK_SET);
-    res = write(trace->f_handle, &trace->nb_threads, sizeof(litl_med_size_t));
-    lseek(trace->f_handle, trace->general_offset, SEEK_SET);
+    __litl_write_flush_thread_header(trace, index, header_size);
   } else {
-    // update the previous offset of the current thread,
-    //   updating the location in the file
-    lseek(trace->f_handle, trace->buffers[index]->offset, SEEK_SET);
-    offset = trace->general_offset - header_size;
-    res = write(trace->f_handle, &offset, sizeof(litl_offset_t));
-    lseek(trace->f_handle, trace->general_offset, SEEK_SET);
+    __litl_write_update_thread_header(trace, index, header_size);
   }
 
   // add an event with offset
   __litl_write_probe_offset(trace, index);
+  lseek(trace->f_handle, trace->general_offset, SEEK_SET);
   if (write(trace->f_handle, trace->buffers[index]->buffer_ptr,
-            __litl_write_get_buffer_size(trace, index)) == -1) {
+	    __litl_write_get_buffer_size(trace, index)) == -1) {
     perror(
-        "Flushing the buffer. Could not write measured data to the trace file!");
+	"Flushing the buffer. Could not write measured data to the trace file!");
     exit(EXIT_FAILURE);
   }
 
@@ -472,8 +504,8 @@ static void __litl_write_allocate_buffer(litl_write_trace_t* trace) {
   if (*pos >= trace->nb_allocated_buffers) {
     // We need to allocate a bigger array of buffers
     void* ptr = realloc(
-        trace->buffers,
-        trace->nb_allocated_buffers * 2 * sizeof(litl_write_buffer_t*));
+	trace->buffers,
+	trace->nb_allocated_buffers * 2 * sizeof(litl_write_buffer_t*));
     if (!ptr) {
       perror("LiTL failed to reallocate memory for threads!\n");
       exit(EXIT_FAILURE);
@@ -482,11 +514,11 @@ static void __litl_write_allocate_buffer(litl_write_trace_t* trace) {
     trace->buffers = ptr;
     unsigned i;
     for (i = trace->nb_allocated_buffers; i < 2 * trace->nb_allocated_buffers;
-        i++) {
+	i++) {
       trace->buffers[i] = malloc(sizeof(litl_write_buffer_t));
       if (!trace->buffers[i]) {
-        perror("Could not allocate memory for a thread\n!");
-        exit(EXIT_FAILURE);
+	perror("Could not allocate memory for a thread\n!");
+	exit(EXIT_FAILURE);
       }
       trace->buffers[i]->already_flushed = 0;
     }
@@ -500,7 +532,8 @@ static void __litl_write_allocate_buffer(litl_write_trace_t* trace) {
 
   trace->buffers[*pos]->buffer_ptr = malloc(
       trace->buffer_size + __litl_get_reg_event_size(LITL_MAX_PARAMS)
-        + __litl_get_reg_event_size(1));
+      + __litl_get_reg_event_size(1));
+
   if (!trace->buffers[*pos]->buffer_ptr) {
     perror("Could not allocate memory buffer for the thread\n!");
     exit(EXIT_FAILURE);
@@ -517,7 +550,10 @@ static void __litl_write_allocate_buffer(litl_write_trace_t* trace) {
  * Allocates an event
  */
 litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
-                               litl_code_t code, int size) {
+			       litl_code_t code, int param_size) {
+  litl_med_size_t index = 0;
+  litl_t*retval = NULL;
+  litl_size_t event_size = LITL_BASE_SIZE + param_size;
 
   if (trace && trace->is_litl_initialized && !trace->is_recording_paused
     && !trace->is_buffer_full) {
@@ -528,15 +564,16 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __litl_write_allocate_buffer(trace);
       p_index = pthread_getspecific(trace->index);
     }
-    litl_med_size_t index = *(litl_med_size_t *) p_index;
+    index = *(litl_med_size_t *) p_index;
 
     litl_write_buffer_t *p_buffer = trace->buffers[index];
 
     // is there enough space in the buffer?
-    if (__litl_write_get_buffer_size(trace, index) < trace->buffer_size) {
+    litl_size_t used_memory= __litl_write_get_buffer_size(trace, index);
+
+    if (used_memory+event_size < trace->buffer_size) {
       // there is enough space for this event
       litl_t* cur_ptr = (litl_t*) p_buffer->buffer;
-      p_buffer->buffer += size;
 
       // fill the event
       cur_ptr->time = litl_get_time();
@@ -545,31 +582,42 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 
       switch (type) {
       case LITL_TYPE_REGULAR:
-        cur_ptr->parameters.regular.nb_params = size;
-        break;
+	cur_ptr->parameters.regular.nb_params = (param_size) / sizeof(litl_param_t);
+	break;
       case LITL_TYPE_RAW:
-        cur_ptr->parameters.raw.size = size;
-        break;
+	cur_ptr->parameters.raw.size = param_size;
+	break;
       case LITL_TYPE_PACKED:
-        cur_ptr->parameters.packed.size = size;
-        break;
+	cur_ptr->parameters.packed.size = param_size;
+	break;
+      case LITL_TYPE_OFFSET:
+	cur_ptr->parameters.offset.nb_params = param_size;
+	break;
       default:
-        fprintf(stderr, "Unknown event type %d\n", type);
-        abort();
+	fprintf(stderr, "Unknown event type %d\n", type);
+	abort();
       }
-      return cur_ptr;
+
+      p_buffer->buffer += __litl_get_gen_event_size(cur_ptr);
+
+      retval = cur_ptr;
+      goto out;
     } else if (trace->allow_buffer_flush) {
 
       // not enough space. flush the buffer and retry
       __litl_write_flush_buffer(trace, index);
-      return __litl_write_get_event(trace, type, code, size);
+      retval =  __litl_write_get_event(trace, type, code, param_size);
+      goto out;
     } else {
       // not enough space, but flushing is disabled so just stop recording
       trace->is_buffer_full = 1;
-      return NULL ;
+      retval = NULL ;
+      goto out;
     }
   }
-  return NULL ;
+
+ out:
+  return retval;
 }
 
 
@@ -579,39 +627,11 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 static litl_t* __litl_write_probe_reg_common(litl_write_trace_t* trace,
 					     litl_code_t code,
 					     unsigned nb_params) {
-  if (!trace->is_litl_initialized || trace->is_recording_paused
-    || trace->is_buffer_full)
-    return NULL;
-
-  if (pthread_getspecific(trace->index) == NULL )
-    __litl_write_allocate_buffer(trace);
-
-  litl_med_size_t index = *(litl_med_size_t *) pthread_getspecific(
-      trace->index);
-  if (__litl_write_get_buffer_size(trace, index) < trace->buffer_size) {
-    litl_t* cur_ptr = (litl_t *) trace->buffers[index]->buffer;
-
-    cur_ptr->time = litl_get_time();
-    cur_ptr->code = code;
-    cur_ptr->type = LITL_TYPE_REGULAR;
-
-    cur_ptr->parameters.regular.nb_params = nb_params;
-    trace->buffers[index]->buffer += LITL_BASE_SIZE + (nb_params * sizeof(litl_param_t));
-
-    return cur_ptr;
-  } else {
-    /* buffer is full */
-    if (trace->allow_buffer_flush) {
-
-      /* flush the buffer to disk */
-      __litl_write_flush_buffer(trace, index);
-      return __litl_write_probe_reg_common(trace, code, nb_params);
-    } else
-      /* stop recording events */
-      trace->is_buffer_full = 1;
-    return NULL;
-  }
-
+  litl_t*retval = __litl_write_get_event(trace,
+					 LITL_TYPE_REGULAR,
+					 code,
+					 nb_params);
+  return retval;
 }
 
 /*
@@ -638,7 +658,7 @@ litl_t* litl_write_probe_reg_1(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with two arguments
  */
 litl_t* litl_write_probe_reg_2(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2) {
+			    litl_param_t param1, litl_param_t param2) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 2);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -651,8 +671,8 @@ litl_t* litl_write_probe_reg_2(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with three arguments
  */
 litl_t* litl_write_probe_reg_3(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3) {
+			    litl_param_t param1, litl_param_t param2,
+			    litl_param_t param3) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 3);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -666,8 +686,8 @@ litl_t* litl_write_probe_reg_3(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with four arguments
  */
 litl_t* litl_write_probe_reg_4(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4) {
+			    litl_param_t param1, litl_param_t param2,
+			    litl_param_t param3, litl_param_t param4) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 4);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -682,9 +702,9 @@ litl_t* litl_write_probe_reg_4(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with five arguments
  */
 litl_t* litl_write_probe_reg_5(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5) {
+			    litl_param_t param1, litl_param_t param2,
+			    litl_param_t param3, litl_param_t param4,
+			    litl_param_t param5) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 5);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -700,9 +720,9 @@ litl_t* litl_write_probe_reg_5(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with six arguments
  */
 litl_t* litl_write_probe_reg_6(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6) {
+			    litl_param_t param1, litl_param_t param2,
+			    litl_param_t param3, litl_param_t param4,
+			    litl_param_t param5, litl_param_t param6) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 6);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -719,10 +739,10 @@ litl_t* litl_write_probe_reg_6(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with seven arguments
  */
 litl_t* litl_write_probe_reg_7(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6,
-                            litl_param_t param7) {
+			    litl_param_t param1, litl_param_t param2,
+			    litl_param_t param3, litl_param_t param4,
+			    litl_param_t param5, litl_param_t param6,
+			    litl_param_t param7) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 7);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -740,10 +760,10 @@ litl_t* litl_write_probe_reg_7(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with eight arguments
  */
 litl_t* litl_write_probe_reg_8(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6,
-                            litl_param_t param7, litl_param_t param8) {
+			    litl_param_t param1, litl_param_t param2,
+			    litl_param_t param3, litl_param_t param4,
+			    litl_param_t param5, litl_param_t param6,
+			    litl_param_t param7, litl_param_t param8) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 8);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -762,11 +782,11 @@ litl_t* litl_write_probe_reg_8(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with nine arguments
  */
 litl_t* litl_write_probe_reg_9(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6,
-                            litl_param_t param7, litl_param_t param8,
-                            litl_param_t param9) {
+			    litl_param_t param1, litl_param_t param2,
+			    litl_param_t param3, litl_param_t param4,
+			    litl_param_t param5, litl_param_t param6,
+			    litl_param_t param7, litl_param_t param8,
+			    litl_param_t param9) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 9);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -786,11 +806,11 @@ litl_t* litl_write_probe_reg_9(litl_write_trace_t* trace, litl_code_t code,
  * Records a regular event with ten arguments
  */
 litl_t* litl_write_probe_reg_10(litl_write_trace_t* trace, litl_code_t code,
-                             litl_param_t param1, litl_param_t param2,
-                             litl_param_t param3, litl_param_t param4,
-                             litl_param_t param5, litl_param_t param6,
-                             litl_param_t param7, litl_param_t param8,
-                             litl_param_t param9, litl_param_t param10) {
+			     litl_param_t param1, litl_param_t param2,
+			     litl_param_t param3, litl_param_t param4,
+			     litl_param_t param5, litl_param_t param6,
+			     litl_param_t param7, litl_param_t param8,
+			     litl_param_t param9, litl_param_t param10) {
   litl_t *cur_ptr = __litl_write_probe_reg_common(trace, code, 10);
   if(cur_ptr) {
     cur_ptr->parameters.regular.param[0] = param1;
@@ -812,51 +832,19 @@ litl_t* litl_write_probe_reg_10(litl_write_trace_t* trace, litl_code_t code,
  * That helps to discover places where the application has crashed
  */
 litl_t* litl_write_probe_raw(litl_write_trace_t* trace, litl_code_t code,
-                          litl_size_t size, litl_data_t data[]) {
-  if (!trace->is_litl_initialized || trace->is_recording_paused
-    || trace->is_buffer_full)
-    return NULL;
-
-  // specify explicitly the end of the string
-  data[size] = '\0';
-  size++;
-
-  if (pthread_getspecific(trace->index) == NULL )
-    __litl_write_allocate_buffer(trace);
-
-  litl_med_size_t i, index;
-  index = *(litl_med_size_t *) pthread_getspecific(trace->index);
-
-  litl_t* cur_ptr = (litl_t *) trace->buffers[index]->buffer;
-  // needs to be done outside of the if statement 'cause of undefined size of
-  //    the string that may cause segfault
-  trace->buffers[index]->buffer += LITL_BASE_SIZE + 7 + size;
-
-  if (__litl_write_get_buffer_size(trace, index) < trace->buffer_size) {
-    cur_ptr->time = litl_get_time();
-    cur_ptr->code = code;
-    cur_ptr->type = LITL_TYPE_RAW;
-    cur_ptr->parameters.raw.size = size;
-    if (size > 0)
-      for (i = 0; i < size; i++)
-        cur_ptr->parameters.raw.data[i] = data[i];
-
-    return cur_ptr;
-
-  } else {
-    /* the buffer is full */
-    if (trace->allow_buffer_flush) {
-      // if there is not enough size we reset back the buffer pointer
-      trace->buffers[index]->buffer -= LITL_BASE_SIZE + 7 + size;
-
-      __litl_write_flush_buffer(trace, index);
-      return litl_write_probe_raw(trace, code, size, data);
-    } else {
-      // this applies only when the flushing is off
-      trace->is_buffer_full = 1;
-      return NULL;
+			  litl_size_t size, litl_data_t data[]) {
+  litl_t* retval = __litl_write_get_event(trace,
+					  LITL_TYPE_RAW,
+					  code,
+					  size+1);
+  if(retval) {
+    litl_size_t i;
+    for (i = 0; i < size; i++) {
+      retval->parameters.raw.data[i] = data[i];
     }
+    retval->parameters.raw.data[size]='\0';
   }
+  return retval;
 }
 
 /*
@@ -867,18 +855,21 @@ void litl_write_finalize_trace(litl_write_trace_t* trace) {
   if(!trace)
     return;
 
-  for (i = 0; i < trace->nb_threads; i++)
+  for (i = 0; i < trace->nb_threads; i++) {
     __litl_write_flush_buffer(trace, i);
+  }
 
   close(trace->f_handle);
   trace->f_handle = -1;
 
-  for (i = 0; i < trace->nb_allocated_buffers; i++)
+  for (i = 0; i < trace->nb_allocated_buffers; i++) {
     if (trace->buffers[i]->tid != 0) {
       free(trace->buffers[i]->buffer_ptr);
+      trace->buffers[i]->buffer_ptr = NULL;
     } else {
       break;
     }
+  }
 
   if (trace->allow_thread_safety) {
     pthread_mutex_destroy(&trace->lock_litl_flush);
